@@ -20,7 +20,7 @@
 #include "filehash.h"
 #include "verify.h"
 
-static char fio_version_string[] = "fio 1.31";
+static char fio_version_string[] = "fio 1.33";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -529,8 +529,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	td->ts.bw_stat[0].min_val = td->ts.bw_stat[1].min_val = ULONG_MAX;
 	td->ddir_nr = td->o.ddir_nr;
 
-	if ((td->o.stonewall || td->o.numjobs > 1 || td->o.new_group)
-	     && prev_group_jobs) {
+	if ((td->o.stonewall || td->o.new_group) && prev_group_jobs) {
 		prev_group_jobs = 0;
 		groupid++;
 	}
